#ifndef MAKER_OCEAN_HPP_INCLUDED
#   define MAKER_OCEAN_HPP_INCLUDED

#   include <gfx/buffer.hpp>
#   include <math/math.hpp>

namespace mak {

class Ocean
{
public:
    Ocean(std::uint8_t dimension, scalar amplitude, scalar gravity, vec2 const& wind, scalar length)
        : m_dimension { dimension }
        , m_buffer_size { dimension + 1U }
        , m_amplitude { amplitude }
        , m_gravity { gravity }
        , m_wind { wind }
        , m_wind_length { len(wind) }
        , m_length { length }
        , m_positions { }
        , m_normals { }
    {
        m_positions.resize(m_buffer_size * m_buffer_size);
        m_normals.resize(m_buffer_size * m_buffer_size);
    }

    gfx::Buffer::PositionsVec const& positions() const { return m_positions; }
    gfx::Buffer::PositionsVec const& normals() const { return m_normals; }

    void update_buffers(scalar t);

    ~Ocean() {}

private:
    vec2 wave_number(vec2i const& coords) const;
    scalar dispersion(scalar wave_length) const;
    scalar philips_spectrum(vec2 const& k, scalar wave_length) const;

    void update_point(vec2i const& coords, scalar t);

    vec2 wave_height(vec2 const& k, scalar wave_length) const;
    vec2 wave_height(vec2 const& k, scalar wave_length, scalar t) const;

    gfx::Buffer::PositionsVec m_positions;
    gfx::Buffer::NormalsVec m_normals;

    std::uint8_t const m_dimension;
    std::uint8_t const m_buffer_size;
    scalar const m_amplitude;
    scalar const m_gravity;
    vec2 const m_wind;
    scalar const m_wind_length;
    scalar const m_length;

};

}

#endif
